from . import clibreboundx
from ctypes import c_double

coordinates = {"JACOBI":0, "BARYCENTRIC":1, "PARTICLE":2} # to use C version's REBX_COORDINATES enum

#function to test whether REBOUND shared library can be located and called correctly
def install_test():
    e = None
    try:
        clibreboundx.install_test.restype = c_double
    except Exception as e:
        return e
    try:
        x = clibreboundx.install_test()
    except Exception as e:
        return e
    try:
        if abs(x-0.17599665767) > 1.e-6:
            return 'Did not integrate to correct value of particles[1].x'
    except Exception as e:
        return e
    
    return e





# -*- coding: utf-8 -*-

#Find suffix
import sysconfig
suffix = sysconfig.get_config_var('EXT_SUFFIX')
if suffix is None:
    suffix = ".so"

#Import shared C library
import os
pymodulespath = os.path.dirname(__file__)
from ctypes import *
clibreboundx = cdll.LoadLibrary(pymodulespath + '/../libreboundx' + suffix)

# Version
__version__ = c_char_p.in_dll(clibreboundx, "rebx_version_str").value.decode('ascii')

# Build
__build__ = c_char_p.in_dll(clibreboundx, "rebx_build_str").value.decode('ascii')
# Check for version

# Githash
__githash__ = c_char_p.in_dll(clibreboundx, "rebx_githash_str").value.decode('ascii')

try:
    moduleversion = pkg_resources.require("reboundx")[0].version
    libreboundxversion = __version__
    if moduleversion != libreboundxversion:
        print("WARNING: python module and libreboundx have different version numbers: '%s' vs '%s'.\n".format(moduleversion, libreboundxversion))
except:
    pass    # this check fails in python 3. Problem with setuptools

# Monkeypatch rebound.Particle to have a params property
import rebound

@property
def params(self):
    params = Params(self)
    return params

rebound.Particle.params = params

from .extras import Extras, Param, Effect
from .tools import coordinates, install_test
from .params import Params
__all__ = ["__version__", "__build__", "__githash__", "Extras", "Param", "Effect", "Params", "coordinates", "install_test"]



import rebound
from collections import MutableMapping
from .extras import Param, Effect
from . import clibreboundx
from ctypes import byref, c_double, c_int, c_int32, c_int64, c_uint, c_uint32, c_longlong, c_char_p, POINTER, cast
from ctypes import c_void_p, memmove, sizeof, addressof
import numpy as np

class Params(MutableMapping):
    def __init__(self, parent):
        self.verbose = 0 # set to 1 to diagnose problems
        self.parent = parent

        # Check rebx instance is attached, otherwise memory isn't allocated.
        if parent.__class__ == rebound.particle.Particle: 
            if not parent._sim.contents.extras:
                raise AttributeError("Need to attach reboundx.Extras instance to simulation before setting"
                        " particle params.")

        #### Update here to add new types. Update only rebx_param_types with new C rebx_types
        #### and self.types for new python types.
        #### ADD ONLY TO THE END OF EACH LIST 
        
        rebx_param_types = [("REBX_TYPE_DOUBLE", float),
                            ("REBX_TYPE_INT", np.int32),
                            ("REBX_TYPE_UINT32", c_uint),
                            ("REBX_TYPE_ORBIT", rebound.Orbit),
                            ("REBX_TYPE_LONGLONG", np.int64),
                           ] # In same order as C enum, with default python types for each
        val = {param_type[0]:value for (value, param_type) in enumerate(rebx_param_types)}

        # tuples of corresponding (python type, dtype, ctype, rebxtype enum value). ONLY ADD TO THE END OF THIS LIST
        # dtype should be same as python type for 'primitive' types, and object for any ctypes structure
        type_tuples = [(int, int, c_int, val["REBX_TYPE_INT"]), 
                       (np.int32, np.int32, c_int, val["REBX_TYPE_INT"]), 
                       (float, float, c_double, val["REBX_TYPE_DOUBLE"]), 
                       (np.float64, np.float64, c_double, val["REBX_TYPE_DOUBLE"]),
                       (c_uint, object, c_uint, val["REBX_TYPE_UINT32"]),
                       (c_uint32, object, c_uint32, val["REBX_TYPE_UINT32"]),
                       (rebound.Orbit, object, rebound.Orbit, val["REBX_TYPE_ORBIT"]),
                       (np.int64, np.int64, c_longlong, val["REBX_TYPE_LONGLONG"]),
                      ]
        
        python_types = [item[0] for item in type_tuples]
        self.ptype = {enum:ptype for (enum, ptype) in enumerate(python_types)} # get python type from enum value
        self.defaultptype = {val[rebxtype]:ptype for (rebxtype, ptype) in rebx_param_types} # get default python type from C rebx type
        self.penum = {ptype:enum for (enum, ptype) in enumerate(python_types)} # get enum from python type
        self.types = {item[0]:(item[1], item[2], item[3]) for item in type_tuples} # (dtype, ctype, rebxtype)from ptype

    def __getitem__(self, key):
        if self.verbose: print("*****")
        clibreboundx.rebx_get_param_node.restype = POINTER(Param)
        node = clibreboundx.rebx_get_param_node(byref(self.parent), c_char_p(key.encode('ascii')))
       
        if node: 
            if self.verbose: print("Parameter {0} found".format(key))
            param = node.contents
            try:
                pythontype = self.ptype[param.python_type]
            except KeyError:
                pythontype = self.defaultptype[param.param_type] # Parameter was set in C, use default

            dtype, ctype, rebxtype = self.types[pythontype]
            if self.verbose:
                print("param.python_type= {0}, pythontype = {1}".format(param.python_type, pythontype))
                print("dtype = {0}, ctype = {1}, rebxtype = {2}".format(dtype, ctype, rebxtype))

            if param.ndim == 0: 
                if self.verbose: print("Scalar")
                data = cast(param.contents, POINTER(ctype))
                try:
                    retval = data.contents.value # cases where scalar is a ctype, so we get python type back 
                    retval = pythontype(retval)
                except AttributeError:
                    if self.verbose: print("Parameter is a Structure")
                    retval = data.contents # cases where scalar is a Structure, e.g., rebound.Orbit
            else:
                if self.verbose: print("Parameter is an Array. ndim = {0}".format(param.ndim))
                if self.verbose: print("Size = {0}".format(param.size))
                ArrayType = ctype*param.size
                data = cast(param.contents, POINTER(ArrayType))
                if dtype == object: # can't use frombuffer with objects. 
                    try:
                        array = np.array([pythontype(data.contents[i]) for i in range(param.size)], dtype=dtype)
                    except TypeError: # for cases where a custom ctypes Structure can't take a class instance in __init__
                        array = np.array([data.contents[i] for i in range(param.size)], dtype=dtype)
                    #Makes copies of the ctypes.Structure instances, but fields share memory with C.
                else:
                    array = np.frombuffer(data.contents, dtype=dtype, count=param.size)
                if param.ndim > 1:
                    shape = [param.shape[i] for i in range(param.ndim)]
                    array = np.reshape(array, shape)
                    if self.verbose: print("Array shape = {0}".format(shape))
                retval = array
        else:
            raise AttributeError("REBOUNDx Error: Parameter '{0}' not found.".format(key))
        
        if self.verbose: print("*****")
        return retval

    def __setitem__(self, key, value):
        if type(value) == list:
            raise AttributeError("Can't assign lists as params. Use numpy arrays instead. See documentation.")
            '''
            value = np.array(value, dtype=object) 
            ''' 
        tempval = value
        try:
            while True: # keep going through nested arrays until we get to a real entry (that we can't index)
                tempval = tempval[0]
                if type(tempval).__name__ == 'void': # this is the type you get if you don't use dtype=object
                                                     # (i.e. when np autogenerates a dtype (which we can't detect)
                    raise AttributeError('When setting a numpy array of objects, have to set dtype=object, '
                            'e.g., ps[1].params["array"] = numpy.array([obj1, obj2, obj3], dtype=object)')
        except:
            pythontype = type(tempval)
            if self.verbose: print("1st array item (to check type) = {0}, type = {1}".format(tempval, pythontype))

        try:
            dtype, ctype, rebxtype = self.types[pythontype]
            if self.verbose: print("ctype = {0}, rebxtype = {1}".format(ctype, rebxtype))
        except KeyError:
            raise AttributeError("REBOUNDx Error: Data type {0} for param '{1}' "
            "not supported.".format(pythontype, key))
        
        clibreboundx.rebx_get_param_node.restype = POINTER(Param)
        nodeptr = clibreboundx.rebx_get_param_node(byref(self.parent), c_char_p(key.encode('ascii')))

        if nodeptr:
            if self.verbose: print("Parameter {0} found.".format(key))
            param = nodeptr.contents
            if param.param_type != rebxtype:
                raise AttributeError("REBOUNDx Error: Can't update param '{0}' with "
                "new type {1}".format(key, pythontype))
            if type(value).__name__ == 'ndarray':
                if self.verbose: print("Param is an array")
                if param.ndim == 0:
                    raise AttributeError("REBOUNDx Error: Cannot update scalar param '{0}' with "
                    "a list or numpy array".format(key))
                else:
                    ArrayType = c_int*param.ndim # shape is an integer array, so we recast it to ctypes array
                    data = cast(param.shape, POINTER(ArrayType))
                    shape = np.frombuffer(data.contents, dtype='int32', count=param.ndim)
                    if self.verbose: print("Array shape = {0}".format(shape))
                    if not np.array_equal(shape, value.shape):
                        raise AttributeError("REBOUNDx Error: Cannot update param '{0}' with a "
                        "list/array of different shape".format(key))
            else:
                if param.ndim > 0:
                    raise AttributeError("REBOUNDx Error: Cannot update param '{0}' "
                    "(a list/array) with scalar".format(key)) 
            val = nodeptr.contents.contents
        else: # parameter not found. Make new one
            if self.verbose: print("Parameter {0} not found. Making new one".format(key))
            if type(value) == np.ndarray:
                if self.verbose: print("Param is an array of dimension {0}".format(value.ndim))
                clibreboundx.rebx_add_param_node.restype = POINTER(Param)
                nodeptr = clibreboundx.rebx_add_param_node(
                        byref(self.parent), c_char_p(key.encode('ascii')), rebxtype, 
                        value.ndim, value.ctypes.shape_as(c_int))
            else: # single value
                if self.verbose: print("Param is a scalar")
                clibreboundx.rebx_add_param_node.restype = POINTER(Param)
                nodeptr = clibreboundx.rebx_add_param_node(
                        byref(self.parent), c_char_p(key.encode('ascii')), rebxtype, 
                        0, None)
            nodeptr.contents.python_type = c_int(self.penum[pythontype])
            if self.verbose:
                print("param.python_type = {0}, pythontype = {1}".format(nodeptr.contents.python_type, pythontype))

        val = cast(nodeptr.contents.contents, POINTER(ctype))
        if type(value) == np.ndarray:
            ArrayType = ctype*value.size
            ctypesarray = ArrayType(*(i for i in value.flatten())) # need to flatten to 1D (final format anyway)
                                                                   # so we don't populate ctypesarray with 
                                                                   # arrays for multidim arrays
            memmove(val, ctypesarray, sizeof(ctype)*value.size)    # COPIES data so that C owns the memory
        else:
            val[0] = value
        
    def __delitem__(self, key):
        success = clibreboundx.rebx_remove_param(byref(self.parent), c_char_p(key.encode('ascii')))
        if not success:
            raise AttributeError("REBOUNDx Error: Parameter '{0}' to delete not found.".format(key))

    def __iter__(self):
        raise AttributeError("REBOUNDx Error: Iterator for params not implemented.")

    def __len__(self):
        current = cast(self.parent.ap, POINTER(Param))
        ctr = 0
        while current:
            ctr += 1
            current = current.contents.next
        return ctr



from . import clibreboundx
from ctypes import Structure, c_double, POINTER, c_int, c_uint, c_long, c_ulong, c_void_p, c_char_p, CFUNCTYPE, byref, c_uint32, c_uint, cast, c_char
import rebound
import reboundx
import warnings

INTEGRATORS = {"implicit_midpoint": 0, "rk4":1, "euler": 2, "none": 3}
  
REBX_BINARY_WARNINGS = [
        ("REBOUNDx Error: Cannot read binary file. Check filename and file contents.", 1),
        ("REBOUNDx Error: Binary file was corrupt. Could not read.", 2),
        ("REBOUNDx Warning: Binary file was saved with a different version of REBOUNDx. Binary format might have changed.", 4),
        ("REBOUNDx Warning: At least one parameter in the binary file was not loaded. Check simulation.", 8),
        ("REBOUNDx Warning: At least one particle's parameters in the binary file were not loaded. Check simulation.", 16),
        ("REBOUNDx Warning: At least one effect and its parameters were not loaded from the binary file. Check simulation.", 32),
        ("REBOUNDx Warning: At least one field in the binary field was not recognized, and not loaded. Probably binary was created with more recent REBOUNDx version than you are using.", 64),
]

class Extras(Structure):
    """
    Main object used for all REBOUNDx operations, tied to a particular REBOUND simulation.
    This is an abstraction of the C struct rebx_extras, with all the C convenience functions
    and functions for adding effects implemented as methods of the class.  
    The fastest way to understand it is to follow the examples at :ref:`ipython_examples`.  
    """

    def __init__(self, sim):
        #first check whether additional_forces or post_timestep_modifications is set on sim.  If so, raise error
        #if cast(sim._additional_forces, c_void_p).value is not None or cast(sim._post_timestep_modifications, c_void_p).value is not None:
        #    raise AttributeError("sim.additional_forces or sim.post_timestep_modifications was already set.  If you want to use REBOUNDx, you need to add custom effects through REBOUNDx.  See https://github.com/dtamayo/reboundx/blob/master/ipython_examples/Custom_Effects.ipynb for a tutorial.")
        
        clibreboundx.rebx_initialize(byref(sim), byref(self)) # Use memory address ctypes allocated for rebx Structure in C
        if not hasattr(sim, "_extras_ref"): # if REBOUNDx wasn't already attached, check for warnings in case additional_forces or ptm were already set.
            sim.process_messages()
        sim._extras_ref = self # add a reference to this instance in sim to make sure it's not garbage collected
        self.custom_effects = {} # dictionary to keep references to custom effects so they don't get garbage collected

    @classmethod
    def from_file(cls, sim, filename):
        """
        Loads REBOUNDx effects along with effect and particle parameters from a binary file.
        
        Arguments
        ---------
        filename : str
            Filename of the binary file.
        
        Returns
        ------- 
        A reboundx.Extras object.
        
        """
        w = c_int(0)
        clibreboundx.rebx_init.restype = POINTER(Extras)
        extrasp = clibreboundx.rebx_init(byref(sim))
        clibreboundx.rebx_create_extras_from_binary_with_messages(extrasp, c_char_p(filename.encode("ascii")),byref(w))
        if (extrasp is None) or (w.value & 1):     # Major error
            raise ValueError(REBX_BINARY_WARNINGS[0])
        for message, value in REBX_BINARY_WARNINGS:  # Just warnings
            if w.value & value and value!=1:
                warnings.warn(message, RuntimeWarning)
        extras = extrasp.contents
        sim.save_messages = 1 # Warnings will be checked within python
        return extras 

    def __del__(self):
        if self._b_needsfree_ == 1:
            clibreboundx.rebx_free_effects(byref(self))
            clibreboundx.rebx_free_params(byref(self))

    def remove_from_simulation(self, sim):
        """
        Disattaches reboundx from simulation, removing all effects.
        """
        del sim._extras_ref
        clibreboundx.rebx_remove_from_simulation(byref(sim))

    @property
    def integrator(self):
        """
        Get or set the intergrator module.

        Available integrators are:

        - ``'implicit_midpoint'`` (default)
        
        Check the online documentation for a full description of each of the integrators. 
        """
        i = self._integrator
        for name, _i in INTEGRATORS.items():
            if i==_i:
                return name
        return i
    @integrator.setter
    def integrator(self, value):
        if isinstance(value, int):
            self._integrator = c_int(value)
        elif isinstance(value, basestring):
            value = value.lower()
            if value in INTEGRATORS: 
                self._integrator = INTEGRATORS[value]
            else:
                raise ValueError("Warning. Integrator not found.")
    
    #######################################
    # Functions for manipulating REBOUNDx effects
    #######################################

    def add(self, name):
        """
        This is the main function for adding effects to simulations.
        :param name: Name of the effect you wish to add.  See http://reboundx.readthedocs.io/en/latest/effects.html for a list of effects.
        :type name: string
        :rtype: Effect structure
        """
        clibreboundx.rebx_add.restype = POINTER(Effect)
        ptr = clibreboundx.rebx_add(byref(self), c_char_p(name.encode('ascii')))
        self.sim.contents.process_messages()
        return ptr.contents

    def add_custom_force(self, function, force_is_velocity_dependent):
        """
        This function allows you to add your own custom python function to REBOUNDx that updates particle accelerations.  
        You need to use this if you want to both use your own custom functions and the built-in REBOUNDx effects.  
        See https://github.com/dtamayo/reboundx/blob/master/ipython_examples/Custom_Effects.ipynb for details and a tutorial on how to use it.
        :param function: Custom Python function for updating particle accelerations.
        :param force_is_velocity_dependent: Whether your custom force depends on the particle velocities.
        :type function: Function
        :type c: bool
        :rtype: Effect structure
        """
        REBX_FORCE = CFUNCTYPE(None, POINTER(rebound.Simulation), POINTER(Effect), POINTER(rebound.Particle), c_int)
        self.custom_effects[function.__name__] = REBX_FORCE(function) # store a ref so it doesn't get garbage collected
        clibreboundx.rebx_add_custom_force.restype = POINTER(Effect)
        ptr = clibreboundx.rebx_add_custom_force(byref(self), c_char_p(function.__name__.encode('ascii')), self.custom_effects[function.__name__], force_is_velocity_dependent)
        return ptr.contents

    def add_custom_operator(self, function):
        """
        This function allows you to add your own custom python function to REBOUNDx that is executed between integrator timesteps.
        You need to use this if you want to both use your own custom functions and the built-in REBOUNDx effects.  
        See https://github.com/dtamayo/reboundx/blob/master/ipython_examples/Custom_Effects.ipynb for details and a tutorial on how to use it.
        :param function: Custom Python function to be executed between timesteps.
        :type function: Function
        :type c: bool
        :rtype: Effect structure 
        """
        REBX_OPERATOR = CFUNCTYPE(None, POINTER(rebound.Simulation), POINTER(Effect), c_double, c_int)
        self.custom_effects[function.__name__] = REBX_OPERATOR(function) # store a ref so it doesn't get garbage collected
        clibreboundx.rebx_add_custom_operator.restype = POINTER(Effect)
        ptr = clibreboundx.rebx_add_custom_operator(byref(self), c_char_p(function.__name__.encode('ascii')), self.custom_effects[function.__name__])
        return ptr.contents
    
    def get_effect(self, name):
        clibreboundx.rebx_get_effect.restype = POINTER(Effect)
        ptr = clibreboundx.rebx_get_effect(byref(self), c_char_p(name.encode('ascii')))
        if ptr:
            return ptr.contents
        else:
            warnings.warn("Parameter {0} not found".format(name), RuntimeWarning)
            return

    def gr_acc(self, acc, C2):
        #sixdoub = c_double*6
        #clibreboundx.rebx_gr_acc.restype = POINTER(sixdoub)
        clibreboundx.rebx_gr_acc(byref(self), acc, c_double(C2))
    def calculate_energy(self):
        clibreboundx.rebx_calculate_energy.restype = c_double
        return clibreboundx.rebx_calculate_energy(self.sim)
    #######################################
    # Input/Output Routines
    #######################################
    def save(self, filename):
        """
        Save the entire REBOUND simulation to a binary file.
        """
        clibreboundx.rebx_output_binary(byref(self), c_char_p(filename.encode("ascii")))

    #######################################
    # Convenience Functions
    #######################################

    def rad_calc_beta(self, G, c, source_mass, source_luminosity, radius, density, Q_pr):
        clibreboundx.rebx_rad_calc_beta.restype = c_double
        return clibreboundx.rebx_rad_calc_beta(c_double(G), c_double(c), c_double(source_mass), c_double(source_luminosity), c_double(radius), c_double(density), c_double(Q_pr))

    def rad_calc_particle_radius(self, G, c, source_mass, source_luminosity, beta, density, Q_pr):
        clibreboundx.rebx_rad_calc_particle_radius.restype = c_double
        return clibreboundx.rebx_rad_calc_particle_radius(c_double(G), c_double(c), c_double(source_mass), c_double(source_luminosity), c_double(beta), c_double(density), c_double(Q_pr))

    def central_force_Acentral(self, p, primary, pomegadot, gamma):
        clibreboundx.rebx_central_force_Acentral.restype = c_double
        Acentral = clibreboundx.rebx_central_force_Acentral(p, primary, c_double(pomegadot), c_double(gamma))
        self.sim.contents.process_messages()
        return Acentral

    # Hamiltonian calculation functions
    def gr_hamiltonian(self, sim, params):
        clibreboundx.rebx_gr_hamiltonian.restype = c_double
        return clibreboundx.rebx_gr_hamiltonian(byref(sim), byref(params))
    
    def gr_potential_hamiltonian(self, sim, params):
        clibreboundx.rebx_gr_potential_hamiltonian.restype = c_double
        return clibreboundx.rebx_gr_potential_hamiltonian(byref(sim), byref(params))
    
    def gr_full_hamiltonian(self, sim, params):
        clibreboundx.rebx_gr_full_hamiltonian.restype = c_double
        return clibreboundx.rebx_gr_full_hamiltonian(byref(sim), byref(params))
    
    def tides_precession_hamiltonian(self, sim, params):
        clibreboundx.rebx_tides_precession_hamiltonian.restype = c_double
        return clibreboundx.rebx_tides_precession_hamiltonian(byref(sim), byref(params))

    def central_force_hamiltonian(self, sim):
        clibreboundx.rebx_central_force_hamiltonian.restype = c_double
        return clibreboundx.rebx_central_force_hamiltonian(byref(sim))
    
    def gravitational_harmonics_hamiltonian(self, sim):
        clibreboundx.rebx_gravitational_harmonics_hamiltonian.restype = c_double
        return clibreboundx.rebx_gravitational_harmonics_hamiltonian(byref(sim))
    
#################################################
# Generic REBOUNDx definitions
#################################################

class Param(Structure): # need to define fields afterward because of circular ref in linked list
    pass    
Param._fields_ =  [("contents", c_void_p),
                        ("hash", c_uint32),
                        ("name", c_char_p),
                        ("param_type", c_int),
                        ("python_type", c_int),
                        ("ndim", c_int),
                        ("shape", POINTER(c_int)),
                        ("strides", POINTER(c_int)),
                        ("size", c_int),
                        ("next", POINTER(Param))]

class Effect(Structure):
    @property 
    def params(self):
        params = Params(self)
        return params

Effect._fields_ = [ ("hash", c_uint32),
                    ("name", c_char_p),
                    ("ap", POINTER(Param)),
                    ("force_as_operator", c_int),
                    ("force", CFUNCTYPE(None, POINTER(rebound.Simulation), POINTER(Effect), POINTER(rebound.Particle), c_int)),
                    ("operator", CFUNCTYPE(None, POINTER(rebound.Simulation), POINTER(Effect), c_double, c_int)),
                    ("operator_order", c_int),
                    ("rebx", POINTER(Extras)),
                    ("next", POINTER(Effect)),
                    ("prev", POINTER(Effect)),
                    ("pad", c_char*100)]

class Param_to_be_freed(Structure):
    pass
Param_to_be_freed._fields_ =  [("param", POINTER(Param)),
                                    ("next", POINTER(Param_to_be_freed))]


# Need to put fields after class definition because of self-referencing
Extras._fields_ =  [("sim", POINTER(rebound.Simulation)),
                    ("effects", POINTER(Effect)),
                    ("params_to_be_freed", POINTER(Param_to_be_freed)),
                    ("_integrator", c_int),]

from .params import Params



from codecs import open
import os
import inspect
import sys 
from distutils import sysconfig

try:
    from setuptools import setup, Extension
    from setuptools.command.build_ext import build_ext as _build_ext
except ImportError:
    print("Installing REBOUNDx requires setuptools.  Do 'pip install setuptools'.")
    sys.exit(1)

suffix = sysconfig.get_config_var('EXT_SUFFIX')
if suffix is None:
    suffix = ".so"

# Try to get git hash
try:
    import subprocess
    ghash = subprocess.check_output(["git", "rev-parse", "HEAD"]).decode("ascii")
    ghash_arg = "-DREBXGITHASH="+ghash
except:
    ghash_arg = "-DREBXGITHASH=b411f436f7fddce00217ebe2cf4d22a4f36f1a3f" #GITHASHAUTOUPDATE

class build_ext(_build_ext):
    def finalize_options(self):
        _build_ext.finalize_options(self)

        try:
            import rebound
        except ImportError:
            print("REBOUNDx did not automatically install REBOUND.  Please let me know if this happens (tamayo.daniel@gmail.com), and try first installing REBOUND (http://rebound.readthedocs.org/en/latest/python_quickstart.html")
            sys.exit(1)
        try:
            version = rebound.__version__ # Added in 2.12.1
        except AttributeError:
            print("REBOUNDx did not automatically install a recent enough version of REBOUND.  Please let me know if this happens (tamayo.daniel@gmail.com), and try upgrading REBOUND.  See 5.3 in http://rebound.readthedocs.org/en/latest/python_quickstart.html")
            sys.exit(1)

        rebdir = os.path.dirname(inspect.getfile(rebound))
        self.include_dirs.append(rebdir)
        self.library_dirs.append(rebdir+'/../')
        for ext in self.extensions:
            ext.runtime_library_dirs.append(rebdir+'/../')
            ext.extra_link_args.append('-Wl,-rpath,'+rebdir+'/../')

from distutils.version import LooseVersion

extra_link_args=[]
if sys.platform == 'darwin':
    from distutils import sysconfig
    vars = sysconfig.get_config_vars()
    vars['LDSHARED'] = vars['LDSHARED'].replace('-bundle', '-shared')
    extra_link_args.append('-Wl,-install_name,@rpath/libreboundx'+suffix)

libreboundxmodule = Extension('libreboundx',
                    sources = [ 'src/central_force.c', 'src/core.c', 'src/euler.c', 'src/gr.c', 'src/gr_full.c', 'src/gr_potential.c', 'src/gravitational_harmonics.c', 'src/input.c', 'src/integrator_implicit_midpoint.c', 'src/modify_mass.c', 'src/modify_orbits_direct.c', 'src/modify_orbits_forces.c', 'src/output.c', 'src/radiation_forces.c', 'src/rebxtools.c', 'src/rk4.c', 'src/tides_precession.c', 'src/tides_synchronous_ecc_damping.c', 'src/track_min_distance.c'],
                    include_dirs = ['src'],
                    library_dirs = [],
                    runtime_library_dirs = ["."],
                    libraries=['rebound'+suffix[:-3]], #take off .so from the suffix
                    define_macros=[ ('LIBREBOUNDX', None) ],
                    extra_compile_args=['-fstrict-aliasing', '-O3','-std=c99', ghash_arg, '-fPIC', '-Wpointer-arith'],
                    extra_link_args=extra_link_args,
                    )

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='reboundx',
    version='2.18.0',
    description='A library for including additional forces in REBOUND',
    long_description=long_description,
    url='http://github.com/dtamayo/reboundx',
    author='Daniel Tamayo',
    author_email='tamayo.daniel@gmail.com',
    license='GPL',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Scientific/Engineering :: Astronomy',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    keywords='astronomy astrophysics nbody integrator',
    packages=['reboundx'],
    cmdclass={'build_ext':build_ext},
    setup_requires=['rebound>=3.2.3', 'numpy'],
    install_requires=['rebound>=3.2.3', 'numpy'],
    tests_require=["numpy","matplotlib"],
    test_suite="reboundx.test",
    ext_modules = [libreboundxmodule],
    zip_safe=False)
